// ----------------------------------------------------------------------------
// FILE NAME      : xilinx_block_ram_be.v
// ----------------------------------------------------------------------------
// Release History
//   VERSION DATE     AUTHOR       DESCRIPTION
//   ------- -------- ------------ --------------------------------------------
//   0.00    11-06-01 John Clayton copied this file from
//                                 "xilinx_block_ram_16.v"
// ----------------------------------------------------------------------------
// PURPOSE: This module instantiates the Dual Port "block RAM" available within
//          the Xilinx Virtex or SpartanII FPGA.
//
//          All of the modules are dual-ported.  If you do not need one of the
//          ports, just connect the inputs to constants, and leave the outputs
//          unconnected.  They will show up as "dangling" in the Xilinx place
//          and route software, but it is only a warning, and the blocks still
//          work fine.  If you don't like that warning, build some new memory
//          blocks which instantiate single-ported constructs.
//
//
//          There are the following modules:
//
//               ramb8_s16be_s16be --  512x16 ( 2 block rams)
//              ramb16_s32be_s32be --  512x32 ( 4 block rams)
//
// These RAMs were put together in order to test the use of byte enables.
// Therefore, these blocks contain byte enable inputs.  Also note that in
// order to initialize the first 32-bit word of "ramb16_s32be_s32" you must
// initialize the first four bytes of each of the four component block rams
// which make up the part.  This is necessary because of the byte enables.
//
// To initialize the contents of the RAMs, try using constraints in your .ucf
// file.  Such as:
//
// INST foo/bar INIT_00=fedcba9876543210;
//
// ----------------------------------------------------------------------------
// Parameters
//   NAME                 RANGE    DESCRIPTION                DEFAULT
//   -------------------- -------- -------------------------- -----------------
//   None.
//
// ----------------------------------------------------------------------------
// REUSE ISSUES
//   Reset Strategy   : None
//   Clock Domains    : sys_clk 
//   Critical Timing  : None.
//   Test Features    : None
//   Asynchronous I/F : None
//   Scan Methodology : N/A
//   Instantiations   : RAMB4_S8_S8 FPGA Virtex Primitive
//   Other        :
// ----------------------------------------------------------------------------


// The following 'include' line must be used with Synplicity to create EDIF.
// The line must be commented for ModelSim.
//`include "C:\synplicity\synplify\lib\xilinx\virtex.v"


module ramb8_s16be_s16be (
dat_o_s16a,
adr_i_s16a,
dat_i_s16a,
be_i_a,
rst_i_s16a,
we_i_s16a,
clk_i_s16a,
dat_o_s16b,
adr_i_s16b,
dat_i_s16b,
be_i_b,
rst_i_s16b,
we_i_s16b,
clk_i_s16b
);
  
// I/O Declarations
output [15:0] dat_o_s16a;            // 16-bit port data output
input [8:0] adr_i_s16a;              // 16-bit port address
input [15:0] dat_i_s16a;             // 16-bit port data input
input [1:0] be_i_a;                  // Byte enables for writing
input rst_i_s16a;                    // 16-bit port reset
input we_i_s16a;                     // 16-bit port write enable
input clk_i_s16a;                    // 16-bit port clock

output [15:0] dat_o_s16b;            // 16-bit port data output
input [8:0] adr_i_s16b;              // 16-bit port address
input [15:0] dat_i_s16b;             // 16-bit port data input
input [1:0] be_i_b;                  // Byte enables for writing
input rst_i_s16b;                    // 16-bit port reset
input we_i_s16b;                     // 16-bit port write enable
input clk_i_s16b;                    // 16-bit port clock


// "Lower BYTE"
RAMB4_S8_S8 b0 (
  .DOA(dat_o_s16a[7:0]),
  .ADDRA(adr_i_s16a[8:0]),
  .CLKA(clk_i_s16a),
  .DIA(dat_i_s16a[7:0]),
  .ENA(be_i_a[0]),
  .RSTA(rst_i_s16a),
  .WEA(we_i_s16a),
  .DOB(dat_o_s16b[7:0]),
  .ADDRB(adr_i_s16b[8:0]),
  .CLKB(clk_i_s16b),
  .DIB(dat_i_s16b[7:0]),
  .ENB(be_i_b[0]),
  .RSTB(rst_i_s16b),
  .WEB(we_i_s16b)
  );


// "Upper BYTE"
RAMB4_S8_S8 b1 (
  .DOA(dat_o_s16a[15:8]),
  .ADDRA(adr_i_s16a[8:0]),
  .CLKA(clk_i_s16a),
  .DIA(dat_i_s16a[15:8]),
  .ENA(be_i_a[1]),
  .RSTA(rst_i_s16a),
  .WEA(we_i_s16a),
  .DOB(dat_o_s16b[15:8]),
  .ADDRB(adr_i_s16b[8:0]),
  .CLKB(clk_i_s16b),
  .DIB(dat_i_s16b[15:8]),
  .ENB(be_i_b[1]),
  .RSTB(rst_i_s16b),
  .WEB(we_i_s16b)
  );

endmodule



// ----------------------------------------------------------------------------
module ramb16_s32be_s32be (
dat_o_s32a,
adr_i_s32a,
dat_i_s32a,
be_i_a,
rst_i_s32a,
we_i_s32a,
clk_i_s32a,
dat_o_s32b,
adr_i_s32b,
dat_i_s32b,
be_i_b,
rst_i_s32b,
we_i_s32b,
clk_i_s32b
);
  
// I/O Declarations
output [31:0] dat_o_s32a;            // 16-bit port data output
input [8:0] adr_i_s32a;              // 16-bit port address
input [31:0] dat_i_s32a;             // 16-bit port data input
input [3:0] be_i_a;                  // Byte enables for writing
input rst_i_s32a;                    // 16-bit port reset
input we_i_s32a;                     // 16-bit port write enable
input clk_i_s32a;                    // 16-bit port clock

output [31:0] dat_o_s32b;            // 16-bit port data output
input [8:0] adr_i_s32b;              // 16-bit port address
input [31:0] dat_i_s32b;             // 16-bit port data input
input [3:0] be_i_b;                  // Byte enables for writing
input rst_i_s32b;                    // 16-bit port reset
input we_i_s32b;                     // 16-bit port write enable
input clk_i_s32b;                    // 16-bit port clock


// "BYTE 0"
RAMB4_S8_S8 b0 (
  .DOA(dat_o_s32a[7:0]),
  .ADDRA(adr_i_s32a[8:0]),
  .CLKA(clk_i_s32a),
  .DIA(dat_i_s32a[7:0]),
  .ENA(be_i_a[0]),
  .RSTA(rst_i_s32a),
  .WEA(we_i_s32a),
  .DOB(dat_o_s32b[7:0]),
  .ADDRB(adr_i_s32b[8:0]),
  .CLKB(clk_i_s32b),
  .DIB(dat_i_s32b[7:0]),
  .ENB(be_i_b[0]),
  .RSTB(rst_i_s32b),
  .WEB(we_i_s32b)
  );

// "BYTE 1"
RAMB4_S8_S8 b1 (
  .DOA(dat_o_s32a[15:8]),
  .ADDRA(adr_i_s32a[8:0]),
  .CLKA(clk_i_s32a),
  .DIA(dat_i_s32a[15:8]),
  .ENA(be_i_a[1]),
  .RSTA(rst_i_s32a),
  .WEA(we_i_s32a),
  .DOB(dat_o_s32b[15:8]),
  .ADDRB(adr_i_s32b[8:0]),
  .CLKB(clk_i_s32b),
  .DIB(dat_i_s32b[15:8]),
  .ENB(be_i_b[1]),
  .RSTB(rst_i_s32b),
  .WEB(we_i_s32b)
  );

// "BYTE 2"
RAMB4_S8_S8 b2 (
  .DOA(dat_o_s32a[23:16]),
  .ADDRA(adr_i_s32a[8:0]),
  .CLKA(clk_i_s32a),
  .DIA(dat_i_s32a[23:16]),
  .ENA(be_i_a[2]),
  .RSTA(rst_i_s32a),
  .WEA(we_i_s32a),
  .DOB(dat_o_s32b[23:16]),
  .ADDRB(adr_i_s32b[8:0]),
  .CLKB(clk_i_s32b),
  .DIB(dat_i_s32b[23:16]),
  .ENB(be_i_b[2]),
  .RSTB(rst_i_s32b),
  .WEB(we_i_s32b)
  );

// "BYTE 3"
RAMB4_S8_S8 b3 (
  .DOA(dat_o_s32a[31:24]),
  .ADDRA(adr_i_s32a[8:0]),
  .CLKA(clk_i_s32a),
  .DIA(dat_i_s32a[31:24]),
  .ENA(be_i_a[3]),
  .RSTA(rst_i_s32a),
  .WEA(we_i_s32a),
  .DOB(dat_o_s32b[31:24]),
  .ADDRB(adr_i_s32b[8:0]),
  .CLKB(clk_i_s32b),
  .DIB(dat_i_s32b[31:24]),
  .ENB(be_i_b[3]),
  .RSTB(rst_i_s32b),
  .WEB(we_i_s32b)
  );

endmodule


